package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.PracticeCourseDto;
import com.ym.mec.biz.dal.dto.PracticeGroupDto;
import com.ym.mec.biz.dal.entity.PracticeGroup;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface PracticeGroupDao extends com.ym.mec.common.dal.BaseDAO<Long, PracticeGroup> {

    /**
     * @describe 获取学生指定日期内预约的陪练课
     * @author Joburgess
     * @date 2020/1/31
     * @param userId: 用户编号
     * @param startDate: 开始时间
     * @param endDate: 结束时间
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     */
    List<PracticeGroup> getUserPracticeCoursesWithDateRange(@Param("userId") Integer userId,
                                                            @Param("startDate") Date startDate,
                                                            @Param("endDate") Date endDate);

    /**
     * @describe 获取学生的免费陪练课
     * @author Joburgess
     * @date 2020/2/24
     * @param userId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     */
    PracticeGroup getUserFreePracticeGroup(@Param("userId") Integer userId);

    /**
     * @describe 批量更新
     * @author Joburgess
     * @date 2020/2/23
     * @param groups:
     * @return int
     */
    int batchUpdate(@Param("groups") List<PracticeGroup> groups);

    /**
     * @describe 统计用户的申请记录
     * @author Joburgess
     * @date 2020/2/3
     * @param userId:
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     */
    int countUserPracticeApplyRecord(@Param("userId") Integer userId);

    /**
     * count陪练课列表数量
     * @param params
     * @return
     */
    int countPracticeGroupByOrgan(Map<String, Object> params);

    /**
     * 获取陪练课列表
     * @param params
     * @return
     */
    List<PracticeGroupDto> findAllByOrgan(Map<String, Object> params);

    /**
     * 获取已上课程数量
     * @param practiceGroupIds
     * @return
     */
    List<Map<Long, Integer>> countPracticeGroupOverCourse(List<String> practiceGroupIds);

    /**
     * 修改老师
     * @param practiceGroupId
     * @param teacherId
     */
    void updateUserId(@Param("practiceGroupId") String practiceGroupId, @Param("teacherId") Integer teacherId);


    /**
     * @describe 获取学生指定的课程组
     * @author Joburgess
     * @date 2020/2/19
     * @param userId:
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     */
    PracticeGroup findUserPracticeGroup(@Param("userId") Integer userId,
                                        @Param("groupId") Long groupId);

    /**
     * @describe 获取学生指定的课程组
     * @author Joburgess
     * @date 2020/2/19
     * @param userId:
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     */
    PracticeGroupDto findUserPracticeGroup2(@Param("userId") Integer userId,
                                        @Param("groupId") Long groupId);

    /**
     * @describe 获取学生最后一次购买的陪练课组
     * @author Joburgess
     * @date 2020/2/17
     * @param userId: 学生编号
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     */
    PracticeGroup findUserLatestPracticeGroup(@Param("userId") Integer userId);

    /**
     * @describe 获取用户购买的陪练课
     * @author Joburgess
     * @date 2020/2/18
     * @param userId: 用户编号
     * @return java.util.List<com.ym.mec.biz.dal.dto.PracticeCourseDto>
     */
    List<PracticeCourseDto> findUserBuyPracticeGroups(@Param("userId") Integer userId);

    /**
     * @describe 获取已经达到截至时间的陪练课程组
     * @author Joburgess
     * @date 2020/2/23
     * @return java.util.List<com.ym.mec.biz.dal.entity.PracticeGroup>
     */
    List<PracticeGroup> findHistoryPracticeGroups();

    /**
     * @describe
     * @author Joburgess
     * @date 2020/2/24
     * @param groupId:
     * @return com.ym.mec.biz.dal.entity.PracticeGroup
     */
    PracticeGroup lockPracticeGroup(@Param("groupId") Integer groupId);

}
