package com.ym.mec.biz.service.impl;


import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.CourseScheduleEvaluate;
import com.ym.mec.biz.dal.entity.PracticeGroup;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.service.CourseScheduleEvaluateService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.http.HttpUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;


@Service
public class CourseScheduleEvaluateServiceImpl extends BaseServiceImpl<Long, CourseScheduleEvaluate> implements CourseScheduleEvaluateService {

    @Autowired
    private CourseScheduleEvaluateDao courseScheduleEvaluateDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private PracticeGroupDao practiceGroupDao;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private SysMessageService sysMessageService;

    @Override
    public BaseDAO<Long, CourseScheduleEvaluate> getDAO() {
        return courseScheduleEvaluateDao;
    }


    @Override
    public boolean addStudyReport(CourseScheduleEvaluate courseScheduleEvaluate) {
        ClassGroup classGroup = classGroupDao.get(courseScheduleEvaluate.getClassGroupId());
        if (classGroup == null) {
            throw new BizException("课程不存在！");
        }
        CourseScheduleEvaluate hasOne = courseScheduleEvaluateDao.findByClassGroupId(courseScheduleEvaluate.getClassGroupId());
        if (hasOne != null) {
            throw new BizException("报告已添加，请勿重复提交");
        }
        courseScheduleEvaluate.setMusicGroupId(classGroup.getMusicGroupId());
        courseScheduleEvaluate.setCreateTime(new Date());
        long num = courseScheduleEvaluateDao.insert(courseScheduleEvaluate);
        if (num <= 0) {
            throw new BizException("报告添加失败，请重试");
        }
        long practiceGroupId = Long.parseLong(classGroup.getMusicGroupId());
        PracticeGroup practiceGroup = practiceGroupDao.get(practiceGroupId);

        Map<Integer, String> push = new HashMap<>(1);
        push.put(practiceGroup.getStudentId(), practiceGroup.getStudentId().toString());
        String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
        String memo = baseUrl + "/#/reportDetail?classGroupId=" + classGroup.getId();
        BasicUserDto teacherInfo = teacherDao.findTeacherInfo(courseScheduleEvaluate.getTeacherId());

        sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.PUSH_STUDY_REPORT, push, null, 0, memo, "STUDENT",
                teacherInfo.getName(),HttpUtil.getSortUrl(memo));
        return true;
    }

    @Override
    public CourseScheduleEvaluate getStudyReport(Integer classGroupId) {
        CourseScheduleEvaluate studyReport = courseScheduleEvaluateDao.findByClassGroupId(classGroupId);
        if (studyReport == null) {
            throw new BizException("报告不存在");
        }
        SysUser user = teacherDao.getUser(studyReport.getStudentId());
        BasicUserDto teacherInfo = teacherDao.findTeacherInfo(studyReport.getTeacherId());
        studyReport.setStudent(user);
        studyReport.setTeacher(teacherInfo);
        long practiceGroupId = Long.parseLong(studyReport.getMusicGroupId());
        PracticeGroup practiceGroup = practiceGroupDao.get(practiceGroupId);
        ClassGroup classGroup = classGroupDao.get(studyReport.getClassGroupId());
        studyReport.setTimes(classGroup.getTotalClassTimes());
        studyReport.setTotalMinutes(classGroup.getTotalClassTimes() * practiceGroup.getSingleClassMinutes());
        return studyReport;

    }
}
